"""
    ZMQ implementation for a simple publish-subscribe system.
    A single subscriber publishes messages to many subscribers.
"""

import threading
import zmq

SHARED_CONTEXT = zmq.Context()

class Publisher:
    
    def __init__(self, port, identifier='', context=SHARED_CONTEXT):
        self.identifier = identifier
        self.socket = context.socket(zmq.PUB)
        self.socket.bind("tcp://*:{0}".format(port))

    def publish(self, message):
        self.socket.send_string(self.identifier + message)

    def notify(self):
        self.socket.send_string(self.identifier + '')


class Subscriber:
    
    def __init__(self, host, port, context=SHARED_CONTEXT):
        self._context = context
        self.host = host
        self.port = port

    def subscribe(self, callback, filters=None, keepalive=lambda:True):
        def configure(socket):
            if filters is None:
                socket.setsockopt_string(zmq.SUBSCRIBE, '')
            else:
                if filters is list:
                    list_filters = filters 
                else:
                    list_filters = [filters]
                for filter in list_filters:
                    socket.setsockopt_string(zmq.SUBSCRIBE, filter)
        
        thread = threading.Thread(
            target=self._target,
            args=(configure, keepalive, callback)
        )
        thread.start()
        return thread
        
    def _target(self, configure, keepalive, callback):
        socket = self._context.socket(zmq.SUB)
        socket.connect("tcp://{0}:{1}".format(self.host, self.port))
        configure(socket)
        while keepalive():
            response = socket.recv_string()
            if not keepalive():
                break
            try:
                callback(response)
            except Exception as e:
                print(e)
                continue
